@@ -275,9 +275,8 @@ class TaskHandler(object):
             else:
                 executable = sys.executable
 
-            env['PATH'] = os.pathsep.join([
-                os.path.dirname(executable), env['PATH']
-            ])
+            env['PATH'] = '{0}:{1}'.format(
+                os.path.dirname(executable), env['PATH'])
             command_args = [executable, '-u', '-m', 'cloudify.dispatch',
                             dispatch_dir]
             self.run_subprocess(command_args,
@@ -741,19 +740,7 @@ class WorkflowHandler(TaskHandler):
     def _update_execution_status(self, status, error=None):
         if self.ctx.local or not self.update_execution_status:
             return
-
-        caught_error = None
-        for _ in range(3):
-            try:
-                return update_execution_status(
-                    self.ctx.execution_id, status, error)
-            except Exception as e:
-                self.ctx.logger.exception(
-                    'Update execution status got unexpected rest error: %s', e)
-                caught_error = e
-                sleep(5)
-        else:
-            raise caught_error
+        return update_execution_status(self.ctx.execution_id, status, error)
 
 
 TASK_HANDLERS = {
