@@ -62,8 +62,11 @@ from cloudify_rest_client.deployment_modifications import (
     DeploymentModificationsClient)
 from cloudify_rest_client.inter_deployment_dependencies import (
     InterDeploymentDependencyClient)
-from cloudify_rest_client.labels import DeploymentsLabelsClient
-from cloudify_rest_client.filters import FiltersClient
+from cloudify_rest_client.labels import (DeploymentsLabelsClient,
+                                         BlueprintsLabelsClient)
+from cloudify_rest_client.filters import (DeploymentsFiltersClient,
+                                          BlueprintsFiltersClient)
+from cloudify_rest_client.workflows import WorkflowsClient
 
 try:
     from requests_kerberos import HTTPKerberosAuth
@@ -487,5 +490,8 @@ class CloudifyClient(object):
         self.cluster_status = ClusterStatusClient(self._client)
         self.inter_deployment_dependencies = InterDeploymentDependencyClient(
             self._client)
+        self.deployments_filters = DeploymentsFiltersClient(self._client)
+        self.blueprints_filters = BlueprintsFiltersClient(self._client)
         self.deployments_labels = DeploymentsLabelsClient(self._client)
-        self.filters = FiltersClient(self._client)
+        self.blueprints_labels = BlueprintsLabelsClient(self._client)
+        self.workflows = WorkflowsClient(self._client)
